

#include "boinc_win.h"
#include "win_util.h"
#include "boinc.h"

#include <vector>
#include <string>
using std::vector;
using std::string;

#include "gui_rpc_client.h"
#include "error_numbers.h"
#include "util.h"
#include "str_util.h"
#include "str_replace.h"
#include "url.h"
#include "version.h"
#include "common_defs.h"

static RPC_CLIENT rpc ;
static CC_STATE ccstate;
static RESULTS globalresults;
static int retval, port;
static MESSAGES messages;
static NOTICES notices;
static char passwd_buf[256], hostname_buf[256],*hostname=0;
static char* passwd = passwd_buf, *p;

static DATA_STRUCT DATA;

//------------------------------------------------------------------------------------------------------
// CPU cmds
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_SetCPU(bool active)
{
	int retval = 0;
	if (active)
		retval = rpc.set_run_mode(RUN_MODE_ALWAYS, 0);
	else
		retval = rpc.set_run_mode(RUN_MODE_NEVER, 0);

	return retval;
}

//------------------------------------------------------------------------------------------------------
// GPU cmds
//-------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_SetGPU(bool active)
{
	int retval = 0;
	if (active)
		retval = rpc.set_gpu_mode(RUN_MODE_ALWAYS, 0);
	else
		retval = rpc.set_gpu_mode(RUN_MODE_NEVER, 0);

	return retval;
}
//------------------------------------------------------------------------------------------------------
// TASK cmds
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetNumberofTask()
{
	return DATA.totWUs;
}
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetTask(int taskNr,char* prjName,char* appName,char* taskName,char* taskState,char* fractionDone)
{
	if (taskNr > DATA.totWUs)
		return -1;

	strcpy(prjName,DATA.WUprojectName[taskNr]);
	strcpy(appName, DATA.WUAppName[taskNr]);
	strcpy(taskName,DATA.WUName[taskNr]);
	strcpy(taskState,DATA.WUstate[taskNr]);
	sprintf(fractionDone,"%d%%",DATA.WUFractionDone[taskNr]);

	return taskNr;
}
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetTaskColor(int taskId)
{
	return DATA.WUColor[taskId];
}
//------------------------------------------------------------------------------------------------------
// PROJECT cmds
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetNumberofProjects()
{
	return DATA.totProjects;
}
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetProjectColor(int prjId)
{
	return DATA.projectColor[prjId];
}
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_GetProject(int projectNr,char* prjName,char* totWUs,char* taskState,char* userName,char* credits)
{
	if (projectNr > DATA.totProjects)
		return -1;

	strcpy(prjName,DATA.projectName[projectNr]);
	sprintf(totWUs,"%d",DATA.projectWUs[projectNr]);
	sprintf(taskState,"%s",DATA.projectState[projectNr]);
	strcpy(userName,DATA.projectUserName[projectNr]);
	strcpy(credits,DATA.projectCredits[projectNr]);

	return projectNr;
}
//------------------------------------------------------------------------------------------------------
// MODE		0=progetto 1=task
// ID		id del task o progetto
// COMMAND per progetto		0=suspend 1=resume 2=nomorework 3=allowwork 4=update 5=reset >50 stopat
// COMMAND per task			0=suspend 1=abort  2=resume
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_SetCommand(int mode,int id,int command)
{
	//progetto
	if (mode == 0)
	{
		PROJECT* t_project = (PROJECT*)ccstate.projects.at(id);

		if (command < 50)
			DATA.projectStopAt[id] = 0;

		if (command == 0)
			rpc.project_op(*(t_project), "suspend");
		else if (command == 1)
		{
			rpc.project_op(*(t_project), "resume");
			//riesuma tutte le WU che possono essere bloccate dalla %
			for (int q=0; q<ccstate.results.size(); q++)
				if (t_project == ccstate.results.at(q)->project)
				{
					RESULT* tResult = ccstate.results.at(q);
					retval = rpc.result_op(*(tResult), "resume");
				}
		}
		else if (command == 2)
			rpc.project_op(*(t_project), "nomorework");
		else if (command == 3)
			rpc.project_op(*(t_project), "allowmorework");
		else if (command == 4)
			rpc.project_op(*(t_project), "update");
		else if (command == 5)
			rpc.project_op(*(t_project), "reset");
		else if (command >= 50)
			DATA.projectStopAt[id] = command;
		else
			return 0;

		
	}
	//task
	else if (mode == 1)
	{
		RESULT* t_task = (RESULT*)ccstate.wus.at(id);
		if (command == 0)
			retval = rpc.result_op(*(t_task), "suspend");
		else if (command == 1)
			retval = rpc.result_op(*(t_task), "abort");
		else if (command == 2)
			retval = rpc.result_op(*(t_task), "resume");
		
	}
	return 1;
}

//------------------------------------------------------------------------------------------------------
// copia tutti i dati nella struttura interna
//------------------------------------------------------------------------------------------------------
int Bridge_CopyData(int i,PROJECT* project,std::vector<RESULT*> results)
{
	//PROJECT
	sprintf(DATA.projectName[i],    "%s",project->project_name.c_str());
	sprintf(DATA.projectUserName[i],"%s",project->user_name.c_str());
	sprintf(DATA.projectCredits[i], "%d (%d)",
		(int)project->user_total_credit,		//TOTALI
		(int)project->user_expavg_credit);	//AVERAGE

	if (project->suspended_via_gui)
	{
		strcpy(DATA.projectState[i],"suspended");
		DATA.projectColor[i] = 1;
	}
	else if (project->dont_request_more_work)
	{	
		strcpy(DATA.projectState[i],"no more work");
		DATA.projectColor[i] = 2;
	}
	else if (project->scheduler_rpc_in_progress)
	{
		strcpy(DATA.projectState[i],"scheduler in progress");
		DATA.projectColor[i]  = 3;
	}
	//se non si presenta nessuno degli altri stati....
	else
	{
		//resetta il progetto...
		strcpy(DATA.projectState[i]," ");
		DATA.projectColor[i] = 0;

		//...se e' definito il SUSPEND AT lo segnala in lista
		if (DATA.projectStopAt[i] > 0 && DATA.projectStopAt[i] < 100)
		{
			DATA.projectColor[i] = 7;
			sprintf(DATA.projectState[i],"stop WUs at %d%%",DATA.projectStopAt[i]);
		}
	}

	//WUs
	DATA.projectWUs[i] = 0;
	for (int q=0; q<results.size(); q++)
	{
		if (results.at(q)->project == project)
		{
			RESULT* tResult = results.at(q);
			strcpy(DATA.WUAppName[DATA.totWUs], tResult->app->name);
			strcpy(DATA.WUprojectName[DATA.totWUs],DATA.projectName[i]);
			sprintf(DATA.WUName[DATA.totWUs],"%s", tResult->name);
			DATA.WUFractionDone[DATA.totWUs] = (int)(double)(tResult->fraction_done * 100.0f);

			//reset sfondo
			strcpy(DATA.WUstate[DATA.totWUs],"ready\0");
			DATA.WUColor[DATA.totWUs] = 10; 
			
			if (tResult->active_task && !tResult->suspended_via_gui)
			{
				strcpy(DATA.WUstate[DATA.totWUs],"running");
				DATA.WUColor[DATA.totWUs] = 0;	
				if (!tResult->active_task_state) 
				{
					sprintf(DATA.WUstate[DATA.totWUs],"Waiting to run");
					DATA.WUColor[DATA.totWUs] = 6;
				}
			}
			else if (tResult->network_wait) 
			{
				strcpy(DATA.WUstate[DATA.totWUs],"waiting for network");
				DATA.WUColor[DATA.totWUs] = 1;	
			}
			else if (tResult->ready_to_report)
			{
				strcpy(DATA.WUstate[DATA.totWUs],"ready to report");
				DATA.WUColor[DATA.totWUs] = 2;
			}
			else if (tResult->suspended_via_gui) 
			{
				strcpy(DATA.WUstate[DATA.totWUs],"suspended by user");
				DATA.WUColor[DATA.totWUs] = 3;
			}
			else if (tResult->project_suspended_via_gui) 
			{
				strcpy(DATA.WUstate[DATA.totWUs],"suspended by project");
				DATA.WUColor[DATA.totWUs] = 4;
			}
			else if (tResult->scheduler_wait) 
			{
				strcpy(DATA.WUstate[DATA.totWUs],"scheduler waiting");
				DATA.WUColor[DATA.totWUs] = 5;
			}
			else
			{
				strcpy(DATA.WUstate[DATA.totWUs],"ready\0");
				DATA.WUColor[DATA.totWUs] = 10; 
			}

			// se nessun altro stato e' definito ma e' definito il suspend schedulizzato...
			if (DATA.projectStopAt[i] > 0 && DATA.projectStopAt[i] < 100)
			{
				if (DATA.WUFractionDone[q] > DATA.projectStopAt[i])
				{
					DATA.WUColor[DATA.totWUs]  = 7;
					sprintf(DATA.WUstate[DATA.totWUs],"stop at %d%%",DATA.projectStopAt[i]);

					//se la frazione ha superato lo stopAt...
					if (DATA.WUFractionDone[DATA.totWUs] >= DATA.projectStopAt[i])
					{
						//invia il suspend al task
						retval = rpc.result_op(*(tResult), "suspend");

						//segnala che e' regolarmente sospesa
						DATA.WUColor[DATA.totWUs] = 3;
						strcpy(DATA.WUstate[DATA.totWUs],"suspended by user");

					}
				}
			}

			++DATA.projectWUs[i];	//contatore WU per progetto
			++DATA.totWUs;			//totale WUs		
		}
	}

	return 0;
}
//------------------------------------------------------------------------------------------------------
// STORE via RESULTS
//------------------------------------------------------------------------------------------------------
EXTERN_C bool Bridge_StoreResults()
{
	DATA.totWUs		 = 0;
	DATA.totProjects = 0;
	globalresults.clear();
	if (rpc.get_results(globalresults,true))
		return false;

	//il numero dei progetti e' sbagliato, size e' il numero dei task
	DATA.totProjects = globalresults.results.size();
	for (int i=0; i<DATA.totProjects; i++)
		Bridge_CopyData(i,globalresults.results.at(i)->project,globalresults.results);

	return 1;
}
//------------------------------------------------------------------------------------------------------
// STORE via CCSTATE
//------------------------------------------------------------------------------------------------------
EXTERN_C bool Bridge_StoreAll()
{
	//chiede lo stato globale
	DATA.totWUs		 = 0;
	DATA.totProjects = 0;
	ccstate.clear();
	if (rpc.get_state(ccstate))
	{
		Bridge_Connect();
		if (rpc.get_state(ccstate))
			return false;
	}

	DATA.totProjects = ccstate.projects.size();
	for (int i=0; i<DATA.totProjects; i++)
		Bridge_CopyData(i,ccstate.projects.at(i),ccstate.results);

	return true;
}
//------------------------------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------------------------------
EXTERN_C int Bridge_Connect()
{
	//recupera la password
#ifdef _WIN32
    chdir_to_data_dir();
#endif
    strcpy(passwd_buf, "");
    read_gui_rpc_password(passwd_buf);
	passwd = &passwd_buf[0];

	//attiva il client
#if defined(_WIN32) && defined(USE_WINSOCK)
    WSADATA wsdata;
    retval = WSAStartup( MAKEWORD( 1, 1 ), &wsdata);
    if (retval) 
	{
		MessageBox(NULL,(LPCSTR)"WinsockInitialize failed",(LPCSTR)"BOINCBridge",MB_OK);
        return 0;
    }
#endif

	// change the following to debug GUI RPC's asynchronous connection mechanism
#if 1
    retval = rpc.init(hostname, port);
    if (retval) {
		MessageBox(NULL,(LPCSTR)"Cant connect to host",(LPCSTR)"BOINCBridge",MB_OK);
        return 0;
    }
#else
    retval = rpc.init_asynch(hostname, 60., false);
    while (1) {
        retval = rpc.init_poll();
        if (!retval) break;
        if (retval == ERR_RETRY) {
            printf("sleeping\n");
            sleep(1);
            continue;
        }
        fprintf(stderr, "can't connect: %d\n", retval);
        return 0;
    }
    printf("connected\n");
#endif

    if (strlen(passwd)) 
	{
        retval = rpc.authorize(passwd);
        if (retval) 
		{
            MessageBox(NULL,(LPCSTR)"Authorization failure",(LPCSTR)"BOINCBridge",MB_OK);
            return 0;
        }
    }
	
	//resetta tutto
	memset(&DATA,0,sizeof(DATA));

	return 1;
}
//------------------------------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------------------------------
EXTERN_C void Bridge_Disconnect()
{
    WSACleanup();
}
//------------------------------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------------------------------
BOOL APIENTRY DllMain( HINSTANCE hinstDLL,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	//globalHInst = hinstDLL;

	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
    return TRUE;
}
//------------------------------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------------------------------

